Attribute VB_Name = "modGeneral"
Option Explicit

'api
Public Const OFS_MAXPATHNAME = 128
Public Const OF_READWRITE = &H2
Public Const TIME_ZONE_ID_INVALID = &HFFFFFFFF
Public Const TIME_ZONE_ID_UNKNOWN = 0
Public Const TIME_ZONE_ID_STANDARD = 1
Public Const TIME_ZONE_ID_DAYLIGHT = 2

Public Type OFSTRUCT
    cBytes As Byte
    fFixedDisk As Byte
    nErrCode As Integer
    Reserved1 As Integer
    Reserved2 As Integer
    szPathName(OFS_MAXPATHNAME) As Byte
End Type
Public Type FileTime
    dwLowDateTime As Long
    dwHighDateTime As Long
End Type
Public Type SYSTEMTIME
    wYear As Integer
    wMonth As Integer
    wDayOfWeek As Integer
    wDay As Integer
    wHour As Integer
    wMinute As Integer
    wSecond As Integer
    wMilliseconds As Integer
End Type
Public Type TIME_ZONE_INFORMATION
   Bias As Long
   StandardName(0 To 63) As Byte ' used to accommodate Unicode strings
   StandardDate As SYSTEMTIME
   StandardBias As Long
   DaylightName(0 To 63) As Byte ' used to accommodate Unicode strings
   DaylightDate As SYSTEMTIME
   DaylightBias As Long
End Type

Public Declare Function WritePrivateProfileString Lib "KERNEL32.DLL" Alias "WritePrivateProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As String, ByVal lpString As String, ByVal lpFileName As String) As Long
Public Declare Function GetTimeZoneInformation Lib "kernel32" (lpTimeZoneInformation As TIME_ZONE_INFORMATION) As Long
Public Declare Function GetWindowsDirectory Lib "KERNEL32.DLL" Alias "GetWindowsDirectoryA" (ByVal lpBuffer As String, ByVal nSize As Long) As Long
Public Declare Function WritePublicProfileString Lib "KERNEL32.DLL" Alias "WritePublicProfileStringA" (ByVal lpApplicationName As String, ByVal lpKeyName As String, ByVal lpString As String, ByVal lpFileName As String) As Long
Public Declare Function LocalFileTimeToFileTime Lib "kernel32" (lpLocalFileTime As FileTime, lpFileTime As FileTime) As Long
Public Declare Function SetFileTime Lib "kernel32" (ByVal hFile As Long, lpCreationTime As FileTime, lpLastAccessTime As FileTime, lpLastWriteTime As FileTime) As Long
Public Declare Function SystemTimeToFileTime Lib "kernel32" (lpSystemTime As SYSTEMTIME, lpFileTime As FileTime) As Long
Public Declare Function CloseHandle Lib "kernel32" (ByVal hObject As Long) As Long
Public Declare Function OpenFile Lib "KERNEL32.DLL" (ByVal lpFileName As String, lpReOpenBuff As OFSTRUCT, ByVal wStyle As Long) As Long
Public Declare Function GetFileTime Lib "KERNEL32.DLL" (ByVal hFile As Long, lpCreationTime As FileTime, lpLastAccessTime As FileTime, lpLastWriteTime As FileTime) As Long
Public Declare Function FileTimeToLocalFileTime Lib "KERNEL32.DLL" (lpFileTime As FileTime, lpLocalFileTime As FileTime) As Long
Public Declare Function FileTimeToSystemTime Lib "KERNEL32.DLL" (lpFileTime As FileTime, lpSystemTime As SYSTEMTIME) As Long
Public Declare Function GetModuleFileName Lib "kernel32" Alias "GetModuleFileNameA" (ByVal hModule As Long, ByVal lpFileName As String, ByVal nSize As Long) As Long
Public Declare Sub apiSleep Lib "KERNEL32.DLL" Alias "Sleep" (ByVal dwMilliseconds As Long)
'imagedll calls
Public Declare Function MakeSureDirectoryPathExists Lib "IMAGEHLP.DLL" (ByVal DirPath As String) As Long

'parses out from a URL the followings
'   return value - the base file name (if sDomain is "http://www.direct-print.com/remail_install.exe" returns "remail_install")
'   sExt         - the file extension (if sDomain is "http://www.direct-print.com/remail_install.exe" returns "exe")
'   sRoot        - the site address (if sDomain is "http://www.direct-print.com/remail_install.exe" returns "http://www.direct-print.com")
Public Function ParseFileBase(ByVal sDomain As String, sExt As String, sRoot As String) As String
    Dim i As Long
    Dim sFile As String
    
    Do While Right$(sDomain, 1) = "/"
        sDomain = Left$(sDomain, Len(sDomain) - 1)
    Loop
    i = InStrRev(sDomain, "/")
    If i > 0 Then
        sRoot = Left$(sDomain, i - 1)
        sFile = Mid$(sDomain, i + 1)
        i = InStrRev(sFile, ".")
        If i > 0 Then
            sExt = Mid$(sFile, i + 1)
            ParseFileBase = Left$(sFile, i - 1)
        End If
    End If
End Function

'ensures a "/" at the end of the string
Public Function WithForeSlash(ByVal sText As String) As String
    If Len(sText) = 0 Then
        WithForeSlash = ""
    Else
        If Right$(sText, 1) = "/" Then
            WithForeSlash = sText
        Else
            WithForeSlash = sText & "/"
        End If
    End If
End Function

'ensures a "\" at the end of the string
Public Function WithSlash(ByVal sText As String) As String
    If Len(sText) = 0 Then
        WithSlash = ""
    Else
        If Right$(sText, 1) = "\" Then
            WithSlash = sText
        Else
            WithSlash = sText & "\"
        End If
    End If
End Function

'retrieves the Windows directory path
Public Function WinDir() As String
    Dim temp As String, x As Long
    temp = Space$(145)
    x = GetWindowsDirectory(temp, 145)
    If InStr(temp, Chr$(0)) > 0 Then temp = Left$(temp, InStr(temp, Chr$(0)) - 1)
    If Right$(temp, 1) <> "\" Then      ' Add \ if necessary
       WinDir = temp + "\"
    Else
       WinDir = temp
    End If
End Function

'writes into an ini file
Public Sub WriteIni(sInifile As String, sSection As String, sEntry As String, vValue As Variant)
    WritePrivateProfileString sSection, sEntry, vValue, sInifile
End Sub

'safely deletes a file
'returns true upon success
Public Function KillSafe(ByVal fPath As String) As Boolean
    'use it instead of the VB Kill function in order to avoid error when the file is not existing
    'even the path is not existing, the file can not be deleted
    'can not be deleted,
    On Error GoTo noKill
    KillSafe = True
    If MakeSureDirectoryPathExists(fPath) Then
        If Dir$(fPath) > "" Then
            If InStr(fPath, "?") = 0 And InStr(fPath, "*") = 0 Then SetAttr fPath, vbNormal
            Kill fPath
        End If
    End If
xnoKill:
    Exit Function
noKill:
    KillSafe = False
    Resume xnoKill
End Function

'returns the filename with the extension from a complete path
Public Function ExtractFile(ByVal PathName As String) As String
    Dim f$, N%
    ' Returns the filename portion of a full pathname

    f$ = PathName

    Do
        N% = InStr(f$, "\")
        If N% > 0 Then f$ = Right$(f$, Len(f$) - N%)
    Loop While N% > 0
    Do
        N% = InStr(f$, "/")
        If N% > 0 Then f$ = Right$(f$, Len(f$) - N%)
    Loop While N% > 0

    ExtractFile = f$

End Function

'returns the filename without the extension from a complete path
Public Function ExtractBase(ByVal PathName As String) As String
    Dim f$, N%
    ' Return the basename portion of a full pathname
    f$ = ExtractFile(PathName)
    N% = InStr(f$, ".")
    If N% > 0 Then f$ = Left$(f$, N% - 1)
    ExtractBase = f$
End Function

'returns the root-path of a file or directory from the complete path
Public Function ExtractPath(PathName As String) As String
    Dim f$, N%
    ' Return the directory path portion of a full pathname

    f$ = PathName

    Do
        N% = InStr(f$, "\")
        If N% > 0 Then f$ = Right$(f$, Len(f$) - N%)
    Loop While N% > 0
    Do
        N% = InStr(f$, "/")
        If N% > 0 Then f$ = Right$(f$, Len(f$) - N%)
    Loop While N% > 0

    ExtractPath = Left$(PathName, Len(PathName) - Len(f$))
End Function

'extracts the file extension from a path
Public Function ExtractExtension(ByVal PathName As String) As String
    Dim f$, N%
    ' Return the file extension portion of a full pathname
    'Note: this function will not work with lower versions of VB than 6.0
    
    f$ = ExtractFile(PathName)
    N% = InStrRev(f$, ".")  'Only from VB 6.0

    If N% > 0 Then f$ = Mid$(f$, N% + 1)

    ExtractExtension = f$

End Function



'retrieves the number of hours the computer's time zone settings is off from UTC
'optionally the ZoneName parameter retrieves the timezone name
'           the daytimeSavings parameters retrieves the daytimesavings state
Public Function HoursToUTC(Optional ZoneName As String, Optional daytimeSavings As Boolean) As Long
    Dim hFile As Long
    Dim lpReOpenBuff As OFSTRUCT
    Dim lpCreationTime As FileTime
    Dim lpLastAccessTime As FileTime
    Dim lpLastWriteTime As FileTime
    Dim locT As FileTime
    Dim sysT As SYSTEMTIME
    Dim tFile As String
    Dim FileDateLocal As Date
    Dim FileDateZulu As Date
    Dim TZ As TIME_ZONE_INFORMATION
    Dim x As Long
    Dim sDayLight As String
    
    Const OF_READWRITE = &H2
    tFile = WinDir() & "Win.ini"
    hFile = OpenFile(tFile, lpReOpenBuff, OF_READWRITE)
    Call GetFileTime(hFile, lpCreationTime, lpLastAccessTime, lpLastWriteTime)
    'take the creation time
    Call FileTimeToLocalFileTime(lpCreationTime, locT)
    Call FileTimeToSystemTime(lpCreationTime, sysT)
    FileDateZulu = DateSerial(sysT.wYear, sysT.wMonth, sysT.wDay) & " " & TimeSerial(sysT.wHour, sysT.wMinute, sysT.wSecond)
    Call FileTimeToSystemTime(locT, sysT)
    FileDateLocal = DateSerial(sysT.wYear, sysT.wMonth, sysT.wDay) & " " & TimeSerial(sysT.wHour, sysT.wMinute, sysT.wSecond)
    
    CloseHandle hFile
    HoursToUTC = DateDiff("h", FileDateZulu, FileDateLocal)
    x = GetTimeZoneInformation(TZ)
    Select Case x
        Case TIME_ZONE_ID_INVALID
            sDayLight = "Invalid zone"
        Case TIME_ZONE_ID_UNKNOWN
            sDayLight = "Unkown zone"
        Case TIME_ZONE_ID_STANDARD
            sDayLight = "No daylight"
        Case TIME_ZONE_ID_DAYLIGHT
            sDayLight = "Daylight"
    End Select
    ZoneName = TZ.StandardName
    If InStr(ZoneName, Chr$(0)) > 0 Then ZoneName = Left$(ZoneName, InStr(ZoneName, Chr$(0)) - 1)
    ZoneName = ZoneName & " " & sDayLight
    daytimeSavings = (sDayLight = "Daylight")
End Function

'returns the file datetime stamp in UTC formatted by the local settings
Public Function FileDateTimeUTC(ByVal sFile As String) As String
    Dim hFile As Long
    Dim lpReOpenBuff As OFSTRUCT
    Dim lpCreationTime As FileTime
    Dim lpLastAccessTime As FileTime
    Dim lpLastWriteTime As FileTime
    Dim locT As FileTime
    Dim sysT As SYSTEMTIME
    
    hFile = OpenFile(sFile, lpReOpenBuff, OF_READWRITE)
    Call GetFileTime(hFile, lpCreationTime, lpLastAccessTime, lpLastWriteTime)
    'take the write time
    Call FileTimeToLocalFileTime(lpLastWriteTime, locT)
    Call FileTimeToSystemTime(lpLastWriteTime, sysT)
    FileDateTimeUTC = DateSerial(sysT.wYear, sysT.wMonth, sysT.wDay) & " " & TimeSerial(sysT.wHour, sysT.wMinute, sysT.wSecond)
    
End Function

'sets the date stamp of a file (all dates, creation, access and modification)
Public Sub SetFileDate(ByVal sFile As String, newDate As Date)
    
    Dim hFile As Long
    Dim lpReOpenBuff As OFSTRUCT
    Dim lpCreationTime As FileTime
    Dim lpLastAccessTime As FileTime
    Dim lpLastWriteTime As FileTime
    Dim locT As FileTime
    Dim sysT As SYSTEMTIME
    Dim oldDate As Date
    
    Const OF_READWRITE = &H2
    
    hFile = OpenFile(sFile, lpReOpenBuff, OF_READWRITE)
    'the following function should return times
    Call GetFileTime(hFile, lpCreationTime, lpLastAccessTime, lpLastWriteTime)

    sysT.wYear = Year(newDate)
    sysT.wMonth = Month(newDate)
    sysT.wDay = Day(newDate)
    sysT.wHour = Hour(newDate)
    sysT.wMinute = Minute(newDate)
    sysT.wSecond = Second(newDate)
    Call SystemTimeToFileTime(sysT, locT)
    Call LocalFileTimeToFileTime(locT, lpCreationTime)
    Call LocalFileTimeToFileTime(locT, lpLastAccessTime)
    Call LocalFileTimeToFileTime(locT, lpLastWriteTime)
    Call SetFileTime(hFile, lpCreationTime, lpLastAccessTime, lpLastWriteTime)
    CloseHandle hFile
End Sub

Public Function inVB() As Boolean
    Dim strFileName As String
    Dim lngCount As Long

    strFileName = String(255, 0)
    lngCount = GetModuleFileName(App.hInstance, strFileName, 255)
    strFileName = Left$(strFileName, lngCount)

    If UCase$(ExtractFile(strFileName)) Like "VB?.EXE" Then
        inVB = True
    Else
        inVB = False
    End If
End Function


